#!/usr/bin/python
#############################################################################
# program:	avrdump.py
# author:	Werner Hoch <werner.ho@gmx.de>
# license:	public domain without any warranty
# decription:	the program reads the binary avr data from stdin
#		the data is assigned to the port descriptions and converted
#		to html
#		data order is:
#		OFFSET: some bytes that have to be rescued before reading the 
#			ram (and the ram mapped ports and registers)
#			with the Offset data I can recover all register, port,..
#		REGISTERS, PORTS, RAM, EEPROM
#############################################################################

VERSION='0.0.1'

import string, sys, cgi

#############################################################################
## constants
#INFILE='/dev/ttyS1'		# used for cgi script
INFILE='/dev/stdin'
OUTFILE='/dev/stdout'

## Register definitions
spl=0x5d
sph=0x5e
sreg=0x5f                      # rescued to ramend-1
temp_reg=29                    # rescued to ramend
zl_reg=30               
zh_reg=31               

#############################################################################
## html constants
#cgi_head="Content-Type: text/html\n\n"		# used for cgi script
cgi_head=""
html_head='''<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0//EN">
<HTML>
<HEAD>
  <TITLE>AVR register-/port-/memory-/eeprom dump</TITLE>
</HEAD>
<BODY>
'''
html_table_start='''<TABLE width=100% BORDER=1>'''
html_tail='''</BODY></HTML>'''
#############################################################################
## variables
#                   ADDR   DESC   BITNAMES                         COMMENT   VALUE
portmap_AT90S2313={ 0x5F:("SREG",['I','T','H','S','V','N','Z','C'],"flag register",0x00),
                    0x5D:("SPL",[],"low(stack pointer)",0x00),
                    0x5B:("GIMSK",['INT1','INT0','','','','','',''],"interrupt mask",0x00),
                    0x5A:("GIFR",['INTF1','INTF0','','','','','',''],"interrupt flags",0x00),
                    0x59:("TIMSK",['TOIE1','OCIE1A','','','TICIE1','','TOIE0',''],"timer interupt mask",0x00),
                    0x58:("TIFR",['TOV1','OCF1A','','','ICF1','','TOV0',''],"timer interrupt flags",0x00),
                    0x55:("MCUCR",['','','SE','SM','ISC11','ISC10','ISC01','ISC00'],"MCU control",0x00),
                    0x53:("TCCR0",['','','','','','CS02','CS01','CS00'],"timer prescale register",0x00),
                    0x52:("TCNT0",[],"timer/counter0 (8bits)",0x00),
                    0x4F:("TCCR1A",['COM1A1','COM1A0','','','','','PWM11','PWM10'],"timer/counter1 control register",0x00),
                    0x4E:("TCCR1B",['ICNC1','ICES1','','','CTC1','CS12','CS11','CS10'],"timer/counter1 control register",0x00),
                    0x4D:("TCNT1H",[],"timer/counter1- high(counter register)",0x00),
                    0x4C:("TCNT1L",[],"timer/counter1- low(counter register)",0x00),
                    0x4B:("OCR1AH",[],"timer/counter1- high(compare register)",0x00),
                    0x4A:("OCR1AL",[],"timer/counter1- low(compare register)",0x00),
                    0x45:("ICR1H",[],"timer/counter1- high(input capture register)",0x00),
                    0x44:("ICR1L",[],"timer/counter1- low(input capture register)",0x00),
                    0x41:("WDTCR",['','','','WDTOE','WDE','WDP2','WDP1','WDP0'],"watchdog timer control",0x00),
                    0x3E:("EEAR",[],"EEPROM adress register",0x00),
                    0x3D:("EEDR",[],"EEPROM data register",0x00),
                    0x3C:("EECR",['','','','','','EEMWE','EEWE','EERE'],"EEPROM control",0x00),
                    0x38:("PORTB",[],"port b output/pullup",0x00),
                    0x37:("DDRB",[],"port b direction register",0x00),
                    0x36:("PINB",[],"port b input",0x00),
                    0x32:("PORTD",[],"port d output/pullup",0x00),
                    0x31:("DDRD",[],"port d direction register",0x00),
                    0x30:("PIND",[],"port d input",0x00),
                    0x2C:("UDR",[],"UART I/O data register",0x00),
                    0x2B:("USR",['RXC','TXC','UDRE','FE','OR','','',''],"UART status register",0x00),
                    0x2A:("UCR",['RXCIE','TXCIE','UDRIE','RXEN','TXEN','CHR9','RXB8','TXB8'],"UART control register",0x00),
                    0x29:("UBRR",[],"UART baud rate register",0x00),
                    0x28:("ACSR",['ACD','','AC0','ACI','ACIE','ACIC','ACIS1','ACIS0'],"analog comparator",0x00) }

#                   ADDR   DESC   BITNAMES                         COMMENT   VALUE
portmap_AT90S8515={ 0x5F:("SREG",['I','T','H','S','V','N','Z','C'],"flag register",0x00),
                    0x5E:("SPH",[],"high(stack pointer)",0x00),
                    0x5D:("SPL",[],"low(stack pointer)",0x00),
                    0x5B:("GIMSK",['INT1','INT0','','','','','',''],"interrupt mask",0x00),
                    0x5A:("GIFR",['INTF1','INTF0','','','','','',''],"interrupt flags",0x00),
                    0x59:("TIMSK",['TOIE1','OCIE1A','OCIE1B','','TICIE1','','TOIE0',''],"timer interupt mask",0x00),
                    0x58:("TIFR",['TOV1','OCF1A','OCF1B','','ICF1','','TOV0',''],"timer interrupt flags",0x00),
                    0x55:("MCUCR",['SRE','SRW','SE','SM','ISC11','ISC10','ISC01','ISC00'],"MCU control",0x00),
                    0x53:("TCCR0",['','','','','','CS02','CS01','CS00'],"timer prescale register",0x00),
                    0x52:("TCNT0",[],"timer/counter0 (8bits)",0x00),
                    0x4F:("TCCR1A",['COM1A1','COM1A0','COM1B1','COM1B0','','','PWM11','PWM10'],"timer/counter1 control register",0x00),
                    0x4E:("TCCR1B",['ICNC1','ICES1','','','CTC1','CS12','CS11','CS10'],"timer/counter1 control register",0x00),
                    0x4D:("TCNT1H",[],"timer/counter1- high(counter register)",0x00),
                    0x4C:("TCNT1L",[],"timer/counter1- low(counter register)",0x00),
                    0x4B:("OCR1AH",[],"timer/counter1- high(compare register a)",0x00),
                    0x4A:("OCR1AL",[],"timer/counter1- low(compare register a)",0x00),
                    0x49:("OCR1BH",[],"timer/counter1- high(compare register b)",0x00),
                    0x48:("OCR1BL",[],"timer/counter1- low(compare register b)",0x00),
                    0x45:("ICR1H",[],"timer/counter1- high(input capture register)",0x00),
                    0x44:("ICR1L",[],"timer/counter1- low(input capture register)",0x00),
                    0x41:("WDTCR",['','','','WDTOE','WDE','WDP2','WDP1','WDP0'],"watchdog timer control",0x00),
                    0x3F:("EEARH",['','','','','','','','EEAR8'],"high(EEPROM adress register)",0x00),
                    0x3E:("EEARL",[],"low(EEPROM adress register)",0x00),
                    0x3D:("EEDR",[],"EEPROM data register",0x00),
                    0x3C:("EECR",['','','','','','EEMWE','EEWE','EERE'],"EEPROM control",0x00),
                    0x3B:("PORTA",[],"port a output/pullup",0x00),
                    0x3A:("DDRA",[],"port a direction register",0x00),
                    0x39:("PINA",[],"port a input",0x00),
                    0x38:("PORTB",[],"port b output/pullup",0x00),
                    0x37:("DDRB",[],"port b direction register",0x00),
                    0x36:("PINB",[],"port b input",0x00),
                    0x35:("PORTC",[],"port c output/pullup",0x00),
                    0x34:("DDRC",[],"port c direction register",0x00),
                    0x33:("PINC",[],"port c input",0x00),
                    0x32:("PORTD",[],"port d output/pullup",0x00),
                    0x31:("DDRD",[],"port d direction register",0x00),
                    0x30:("PIND",[],"port d input",0x00),
                    0x2F:("SPDR",[],"SPI data register",0x00),
                    0x2E:("SPSR",['SPIF','WCOL','','','','','',''],"SPI status register",0x00),
                    0x2D:("SPCR",['SPIE','SPE','DORD','MSTR','CPOL','CPHA','SPR1','SPR0'],"SPI control register",0x00),
                    0x2C:("UDR",[],"UART I/O data register",0x00),
                    0x2B:("USR",['RXC','TXC','UDRE','FE','OR','','',''],"UART status register",0x00),
                    0x2A:("UCR",['RXCIE','TXCIE','UDRIE','RXEN','TXEN','CHR9','RXB8','TXB8'],"UART control register",0x00),
                    0x29:("UBRR",[],"UART baud rate register",0x00),
                    0x28:("ACSR",['ACD','','AC0','ACI','ACIE','ACIC','ACIS1','ACIS0'],"analog comparator",0x00) }

#                   ADDR   DESC   BITNAMES                         COMMENT   VALUE
portmap_AT90S8535={ 0x5F:("SREG",['I','T','H','S','V','N','Z','C'],"flag register",0x00),
                    0x5E:("SPH",[],"high(stack pointer)",0x00),
                    0x5D:("SPL",[],"low(stack pointer)",0x00),
                    0x5B:("GIMSK",['INT1','INT0','','','','','',''],"interrupt mask",0x00),
                    0x5A:("GIFR",['INTF1','INTF0','','','','','',''],"interrupt flags",0x00),
                    0x59:("TIMSK",['OCFIE2','TOIE2','TICIE1','OCIE1A','OCIE1B','TOIE1','','TOIE0'],"timer interupt mask",0x00),
                    0x58:("TIFR",['OCF2','TOV2','ICF1','OCF1A','OCF1B','TOV1','','TOV0'],"timer interrupt flags",0x00),
                    0x55:("MCUCR",['','SE','SM1','SM0','ISC11','ISC10','ISC01','ISC00'],"MCU control",0x00),
                    0x54:("MCUSR",['','','','','','','EXTRF','PORF'],"MCU status register",0x00),
                    0x53:("TCCR0",['','','','','','CS02','CS01','CS00'],"timer prescale register",0x00),
                    0x52:("TCNT0",[],"timer/counter0 (8bits)",0x00),
                    0x4F:("TCCR1A",['COM1A1','COM1A0','COM1B1','COM1B0','','','PWM11','PWM10'],"timer/counter1 control register",0x00),
                    0x4E:("TCCR1B",['ICNC1','ICES1','','','CTC1','CS12','CS11','CS10'],"timer/counter1 control register",0x00),
                    0x4D:("TCNT1H",[],"timer/counter1- high(counter register)",0x00),
                    0x4C:("TCNT1L",[],"timer/counter1- low(counter register)",0x00),
                    0x4B:("OCR1AH",[],"timer/counter1- high(compare register a)",0x00),
                    0x4A:("OCR1AL",[],"timer/counter1- low(compare register a)",0x00),
                    0x49:("OCR1BH",[],"timer/counter1- high(compare register b)",0x00),
                    0x48:("OCR1BL",[],"timer/counter1- low(compare register b)",0x00),
                    0x47:("ICR1H",[],"timer/counter1- high(input capture register)",0x00),
                    0x46:("ICR1L",[],"timer/counter1- low(input capture register)",0x00),
                    0x45:("TCCR2",['','PWM2','COM21','COM20','CTC2','CS22','CS21','CS20'],"timer/counter2 control",0x00),
                    0x44:("TCNT2",[],"timer/counter2",0x00),
                    0x43:("OCR2",[],"timer/counter2 output compare register",0x00),
                    0x42:("ASSR",['','','','','AS2','TCN2UB','OCR2UB','TCR2UB'],"asynchronous status register",0x00),
                    0x41:("WDTCR",['','','','WDTOE','WDE','WDP2','WDP1','WDP0'],"watchdog timer control",0x00),
                    0x3F:("EEARH",['','','','','','','','EEAR8'],"high(EEPROM adress register)",0x00),
                    0x3E:("EEARL",[],"low(EEPROM adress register)",0x00),
                    0x3D:("EEDR",[],"EEPROM data register",0x00),
                    0x3C:("EECR",['','','','','EERIE','EEMWE','EEWE','EERE'],"EEPROM control",0x00),
                    0x3B:("PORTA",[],"port a output/pullup",0x00),
                    0x3A:("DDRA",[],"port a direction register",0x00),
                    0x39:("PINA",[],"port a input",0x00),
                    0x38:("PORTB",[],"port b output/pullup",0x00),
                    0x37:("DDRB",[],"port b direction register",0x00),
                    0x36:("PINB",[],"port b input",0x00),
                    0x35:("PORTC",[],"port c output/pullup",0x00),
                    0x34:("DDRC",[],"port c direction register",0x00),
                    0x33:("PINC",[],"port c input",0x00),
                    0x32:("PORTD",[],"port d output/pullup",0x00),
                    0x31:("DDRD",[],"port d direction register",0x00),
                    0x30:("PIND",[],"port d input",0x00),
                    0x2F:("SPDR",[],"SPI data register",0x00),
                    0x2E:("SPSR",['SPIF','WCOL','','','','','',''],"SPI status register",0x00),
                    0x2D:("SPCR",['SPIE','SPE','DORD','MSTR','CPOL','CPHA','SPR1','SPR0'],"SPI control register",0x00),
                    0x2C:("UDR",[],"UART I/O data register",0x00),
                    0x2B:("USR",['RXC','TXC','UDRE','FE','OR','','',''],"UART status register",0x00),
                    0x2A:("UCR",['RXCIE','TXCIE','UDRIE','RXEN','TXEN','CHR9','RXB8','TXB8'],"UART control register",0x00),
                    0x29:("UBRR",[],"UART baud rate register",0x00),
                    0x28:("ACSR",['ACD','','AC0','ACI','ACIE','ACIC','ACIS1','ACIS0'],"analog comparator",0x00),
                    0x27:("ADMUX",['','','','','','MUX2','MUX1','MUX0'],"ADC-multiplexer",0x00),
                    0x26:("ADCSR",['ADEN','ADSC','ADFR','ADIF','ADIE','ADPS2','ADPS1','ADPS0'],"ADC status register",0x00),
                    0x25:("ADCH",['','','','','','','ADC9','ADC8'],"high(ADC data register",0x00),
                    0x24:("ADCL",[],"low(ADC data register)",0x00) }

#                   ADDR   DESC   BITNAMES                         COMMENT   VALUE
portmap_AT90S4433={ 0x5F:("SREG",['I','T','H','S','V','N','Z','C'],"flag register",0x00),
                    0x5D:("SPL",[],"low(stack pointer)",0x00),
                    0x5B:("GIMSK",['INT1','INT0','','','','','',''],"interrupt mask",0x00),
                    0x5A:("GIFR",['INTF1','INTF0','','','','','',''],"interrupt flags",0x00),
                    0x59:("TIMSK",['TOIE1','OCIE1','','','TICIE1','','TOIE0',''],"timer interupt mask",0x00),
                    0x58:("TIFR",['TOV1','OCF1','','','ICF1','','TOV0',''],"timer interrupt flags",0x00),
                    0x55:("MCUCR",['','','SE','SM','ISC11','ISC10','ISC01','ISC00'],"MCU control",0x00),
                    0x54:("MCUSR",['','','','','WDRF','BORF','EXTRF','PORF'],"MCU status register",0x00),
                    0x53:("TCCR0",['','','','','','CS02','CS01','CS00'],"timer prescale register",0x00),
                    0x52:("TCNT0",[],"timer/counter0 (8bits)",0x00),
                    0x4F:("TCCR1A",['COM11','COM10','','','','','PWM11','PWM10'],"timer/counter1 control register",0x00),
                    0x4E:("TCCR1B",['ICNC1','ICES1','','','CTC1','CS12','CS11','CS10'],"timer/counter1 control register",0x00),
                    0x4D:("TCNT1H",[],"timer/counter1- high(counter register)",0x00),
                    0x4C:("TCNT1L",[],"timer/counter1- low(counter register)",0x00),
                    0x4B:("OCR1H",[],"timer/counter1- high(compare register)",0x00),
                    0x4A:("OCR1L",[],"timer/counter1- low(compare register)",0x00),
                    0x45:("ICR1H",[],"timer/counter1- high(input capture register)",0x00),
                    0x44:("ICR1L",[],"timer/counter1- low(input capture register)",0x00),
                    0x41:("WDTCR",['','','','WDTOE','WDE','WDP2','WDP1','WDP0'],"watchdog timer control",0x00),
                    0x3E:("EEAR",[],"EEPROM adress register",0x00),
                    0x3D:("EEDR",[],"EEPROM data register",0x00),
                    0x3C:("EECR",['','','','','EERIE','EEMWE','EEWE','EERE'],"EEPROM control",0x00),
                    0x38:("PORTB",[],"port b output/pullup",0x00),
                    0x37:("DDRB",[],"port b direction register",0x00),
                    0x36:("PINB",[],"port b input",0x00),
                    0x35:("PORTC",[],"port c output/pullup",0x00),
                    0x34:("DDRC",[],"port c direction register",0x00),
                    0x33:("PINC",[],"port c input",0x00),
                    0x32:("PORTD",[],"port d output/pullup",0x00),
                    0x31:("DDRD",[],"port d direction register",0x00),
                    0x30:("PIND",[],"port d input",0x00),
                    0x2F:("SPDR",[],"SPI data register",0x00),
                    0x2E:("SPSR",['SPIF','WCOL','','','','','',''],"SPI status register",0x00),
                    0x2D:("SPCR",['SPIE','SPE','DORD','MSTR','CPOL','CPHA','SPR1','SPR0'],"SPI control register",0x00),
                    0x2C:("UDR",[],"UART I/O data register",0x00),
                    0x2B:("USR",['RXC','TXC','UDRE','FE','OR','','',''],"UART status register",0x00),
                    0x2A:("UCR",['RXCIE','TXCIE','UDRIE','RXEN','TXEN','CHR9','RXB8','TXB8'],"UART control register",0x00),
                    0x29:("UBRR",[],"UART baud rate register",0x00),
                    0x28:("ACSR",['ACD','','AC0','ACI','ACIE','ACIC','ACIS1','ACIS0'],"analog comparator",0x00),
                    0x27:("ADMUX",['','ADCBG','','','','MUX2','MUX1','MUX0'],"ADC-multiplexer",0x00),
                    0x26:("ADCSR",['ADEN','ADSC','ADFR','ADIF','ADIE','ADPS2','ADPS1','ADPS0'],"ADC status register",0x00),
                    0x25:("ADCH",['','','','','','','ADC9','ADC8'],"high(ADC data register",0x00),
                    0x24:("ADCL",[],"low(ADC data register)",0x00) }

# template line                    0x : ("",['','','','','','','',''],"",0x00),

portmaps={'AT90S2313': portmap_AT90S2313,
          'AT90S8515': portmap_AT90S8515,
          'AT90S8535': portmap_AT90S8535,
          'AT90S4433': portmap_AT90S4433}

#############################################################################
## functions
def tobin(n):
    hex=['0000','0001','0010','0011','0100','0101','0110','0111',
         '1000','1001','1010','1011','1100','1101','1110','1111']
    s=hex[n/16]+hex[n%16]+'b'
    return s

def tochar(n):
    if n < 128 and n > 31:
        letter = chr(n)
        if letter == "<":
            ret= "&lt;"
        elif letter == ">":
            ret = "&gt;"
        elif letter == "&":
            ret = "&amp;"
        else:
            ret = letter
        return chr(n)
    else:
        return "."

def num2str(n):
    s=tobin(n)
    c=tochar(n)
    return   '0x%02X , '%n + s + ' , %i'%n + ' , "' + c + '"'

def cgi_write():
    f_h.write(cgi_head)
    f_h.write(html_head)
    f_h.write('''<h1> avr-dump: device "'''+ device+ '''" </h1>''')
    f_h.write('''<h3> register </h3>''')
    f_h.write(html_table_start)
    for i in range(REGISTERS/4):
        f_h.write('<tr>\n')
        for j in range(4):
            pos = addr = i + REGISTERS/4*j
            f_h.write('<td bgcolor="#FFAAAA">' +'r%i'%addr + '</td>')
            datum=data_all[pos]
            f_h.write('<td>' + num2str(datum) + '</td>')
        f_h.write('</tr>\n')
    f_h.write('</table> ')

    f_h.write('''<h3> ports </h3>''')
    f_h.write(html_table_start)
    for i in range(PORTS):
        addr = pos = i + REGISTERS
        if portmap.has_key(addr):	## skip all reserved ports
            name, bits, descr, value = portmap[addr]
            f_h.write('<tr>\n')
            f_h.write('<td bgcolor="#AAFFAA">' +'0x%02X'%addr + '</td>')
            f_h.write('<td>' + name + '</td>')
            datum=data_all[pos]
            s=tobin(datum)
            if len(bits) > 7 :
                for j in range(8):
                    if bits[j] != '':
                        f_h.write('<td>' + bits[j] +'='+ s[j] +'</td>')
                    else:
                        f_h.write('<td>' + '</td>')
            else:
                f_h.write('<td colspan="8" align="center">'
                          + num2str(datum) + '</td>')
            f_h.write('<td>' + descr + '</td>')
            f_h.write('</tr>\n')
    f_h.write('</table>')

    f_h.write('''<h3> memory </h3>''')
    f_h.write(html_table_start)
    for i in range(RAM/4):
        f_h.write('<tr>\n')
        for j in range(4):
            pos = addr = REGISTERS + PORTS + i + RAM/4*j
            color='<td bgcolor="#AAAAFF">'
            if  addr == stackpointer :
                color = '<td bgcolor="#FFAAAA">'
            f_h.write(color +'0x%02X'%addr + '</td>')
            datum=data_all[pos]
            f_h.write('<td>' + num2str(datum) + '</td>')
        f_h.write('</tr>\n')
    f_h.write('</table> ')

    f_h.write('''<h3> eeprom </h3>''')
    f_h.write(html_table_start)
    for i in range(EEPROM/4):
        f_h.write('<tr>\n')
        for j in range(4):
            pos = addr = i + EEPROM/4*j
            f_h.write('<td bgcolor="#FFFFAA">' +'0x%02x'%addr + '</td>')
            datum=data_eepr[pos]
            f_h.write('<td>' + num2str(datum) + '</td>')
        f_h.write('</tr>\n')
    f_h.write('</table> ')
    f_h.write(html_tail)

def data_read(size):
    f=open(INFILE,"rb")
    d_temp=f.read(size)
    d=[]
    for i in xrange(len(d_temp)):
        d.append(ord(d_temp[i]))
    f.close()
    return d


#############################################################################
## MAIN
try:
    device=sys.argv[1]
except:
    print "avrdump.py version " + VERSION
    print "Bad arguments, usage is: ", sys.argv[0] ,"device < input > output"
    sys.exit()

if portmaps.has_key(device):
    portmap=portmaps[device]
else:
    print "Bad arguments, usage is: ", sys.argv[0] ,"device < input > output"
    print "no such device available"
    print "devices: ", portmaps.keys()
    sys.exit()

f_h=open(OUTFILE,"wt")

if device == "AT90S8515" or device == "AT90S8535":
    OFFSET, REGISTERS, PORTS, RAM, EEPROM = 3, 32, 64, 512, 512
    data=data_read(OFFSET+ REGISTERS+ PORTS+ RAM+ EEPROM)
    ## data order is: OFFSET, REGS, PORTS, RAM, EEPROM
    data_all=data[OFFSET:OFFSET+REGISTERS+PORTS+RAM]
    data_eepr=data[OFFSET+PORTS+REGISTERS+RAM:OFFSET+PORTS+REGISTERS+RAM+EEPROM]
    ## get ZH, ZL and SREG from the stack
    stackpointer=256*(data_all[sph]) + (data_all[spl])
    ramend=REGISTERS+ PORTS + RAM -1
    data_all[sreg]=data_all[ramend-1]
    data_all[temp_reg]=data_all[ramend]
    data_all[zl_reg]=data_all[stackpointer+2]
    data_all[zh_reg]=data_all[stackpointer+1]
    ## move the values from the offset back to stack
    data_all[stackpointer+0]=data[0]
    data_all[stackpointer+1]=data[1]
    data_all[stackpointer+2]=data[2]
     ## ajust the stackpointer
    stackpointer = stackpointer + 2
    data_all[spl] = stackpointer % 256
    data_all[sph] = stackpointer / 256
    ## html output
    cgi_write()

elif device == "AT90S2313" or device == "AT90S4433":
    OFFSET, REGISTERS, PORTS, RAM, EEPROM = 2, 32, 64, 128, 128
    if device == "AT90S4433":
        OFFSET, REGISTERS, PORTS, RAM, EEPROM = 2, 32, 64, 128, 256
    data=data_read(EEPROM+OFFSET+RAM+REGISTERS+PORTS)
    ## data order is: OFFSET, REGS, PORTS, RAM, EEPROM
    data_all=data[OFFSET:OFFSET+REGISTERS+PORTS+RAM]
    data_eepr=data[OFFSET+PORTS+REGISTERS+RAM:OFFSET+PORTS+REGISTERS+RAM+EEPROM]
    ## get ZH, ZL and SREG from the stack
    stackpointer= data_all[spl]
    ramend=REGISTERS+ PORTS + RAM -1
    data_all[sreg]=data_all[ramend-1]
    data_all[temp_reg]=data_all[ramend]
    data_all[zl_reg]=data_all[stackpointer+2]
    data_all[zh_reg]=data_all[stackpointer+1]
    ## correct the bytes inside the offset, put the two values back to the stack
    data_all[stackpointer+1]=data[0]
    data_all[stackpointer+2]=data[1]
    ## ajust the stack, now only the [sp+1] value is wrong
    stackpointer = stackpointer + 2
    data_all[spl] = stackpointer % 256
    ## html output    
    cgi_write()

f_h.close()
