;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; program:	dump8515.asm
;; author:	Werner Hoch <werner.ho@gmx.de
;; license:	public domaine without any warranty
;; description:	send all registers, ports, ram, eeprom to RS232
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

.include "8515def.inc"		; 8535 should work too

.def	temp1	=r17

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; interrupt vector table

.org 0x000                    ; kommt ganz an den Anfang des Speichers
        rjmp	MAIN		; main programm
        rjmp	EXT_INT0        ; IRQ0 Handler
        reti		;rjmp	EXT_INT1        ; IRQ1 Handler
        reti		;rjmp	TIM1_CAPT       ; Timer1 Capture Handler
        reti		;rjmp	TIM1_COMPA      ; Timer1 compare Handler
	reti		;rjmp	TIM1_COMPB
        reti		;rjmp	TIM1_OVF        ; Timer1 Overflow Handler
	reti		;rjmp	TIM0_OVF        ; Timer0 Overflow Handler
        reti		;rjmp	SPI_STC         ; SPI Transfer Complete Handler
        reti		;rjmp	UART_RXC        ; UART RX Complete Handler
        reti		;rjmp	UART_DRE        ; UDR Empty Handler
        reti		;rjmp	UART_TXC        ; UART TX Complete Handler
        reti		;rjmp	ANA_COMP        ; Analog Comparator Handler

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; INT0 external interrupt on PD2
EXT_INT0:
	sts	RAMEND, r29		; rescue the temporary reg I'll use
	in	r29, SREG		; rescue SREG
	sts	(RAMEND-1), r29
	in	r29, SPL		; rescue 3 memory locations for 
	subi	r29, 0x03		; ZL, ZH and EEDR, EEARL, EEARH
	out	SPL, r29
	brcc	_DUMP1			; no overflow of SPL
	in	r29, SPH
	dec	r29
	out	SPH, r29
_DUMP1:					; rescue top of stack (3 bytes)
	pop	r29			; rescue [SP-2]
_DUMP2:	sbis	USR, UDRE		; wait till UDR empty
	rjmp	_DUMP2
	out	UDR, r29
	pop	r29			; rescue [SP-1]
_DUMP3:	sbis	USR, UDRE		; wait till UDR empty
	rjmp	_DUMP3
	out	UDR, r29
	pop	r29			; rescue [SP]
_DUMP4:	sbis	USR, UDRE		; wait till UDR empty
	rjmp	_DUMP4
	out	UDR, r29

	;; send registers, ports, and ram
	push	ZL		
	push	ZH		
	eor	ZH, ZH
	eor	ZL, ZL
_DUMP6:	ld	r29, Z+
_DUMP7:	sbis	USR, UDRE		; wait till UDR empty
	rjmp	_DUMP7
	out	UDR, r29
	cpi	ZH, high(RAMEND+1)	; +1 because of postincrement
	brne	_DUMP6
	cpi	ZL, low(RAMEND+1)
	brne	_DUMP6
	pop	ZH
	pop	ZL

	;; lets read the EEPROM now
	in	r29, EEARH		; rescue all EE-ports
	push	r29
	in	r29, EEARL
	push	r29
	in	r29, EEDR
	push	r29
	eor	r29, r29		; start at EEAR=0		
	out	EEARH, r29
	out	EEARL, r29
_DUMP8:	sbi	EECR, EERE
	in	r29, EEDR
_DUMP9:	sbis	USR, UDRE		; wait till UDR empty
	rjmp	_DUMP9
	out	UDR, r29
	in	r29, EEARH		; check whether we reached the end
	cpi	r29, high(E2END)
	brne	_DUMP_EEINC
	in	r29, EEARL
	cpi	r29, low(E2END)
	breq	_DUMP_EEEND
_DUMP_EEINC:
	in	r29, EEARL
	inc	r29			; increase EE-adress registers
	out	EEARL, r29
	brne	_DUMP8
	in	r29, EEARH
	inc	r29
	out	EEARH, r29
	rjmp	_DUMP8

_DUMP_EEEND:
	pop	r29			; restore EE-ports from stack
	out	EEDR, r29
	pop	r29
	out	EEARL, r29
	pop	r29
	out	EEARH, r29
	lds	r29, (RAMEND-1)		; restore SREG and r29
	out	SREG, r29
	lds	r29, RAMEND
	reti

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; MAIN PROGRAMM
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
MAIN:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; stack pointer to highest address in internal SRAM
;; NOTE: the 2 highest RAMs are for the dump routine	
	ldi	temp1, low(RAMEND-2)	 
	out	SPL, temp1
	ldi	temp1, high(RAMEND-2)	 
	out	SPH, temp1

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; RS232 Init
	sbi	UCR, TXEN
	ldi	temp1, 25	;  set Baudrate to 9600 @ 4MHz
	out	UBRR, temp1

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Init interrupt
	ldi	temp1, (1<<INT0)		;activate INT0
	out	GIMSK, temp1
	ldi	temp1, (1<<ISC01)+(1<<ISC00)	;on rising edge
	out	MCUCR, temp1
	sei					;enable global int

;; load test patterns into some regs 
	ldi	r29, 0x33	
	ldi	r30, 0x22	; ZL
	ldi	r31, 0x11	; ZH
	ldi	temp1,0xa0
	push	temp1
	inc	temp1
	push	temp1
	inc	temp1
	push	temp1
	inc	temp1
	push	temp1
	inc	temp1
	push	temp1
	inc	temp1
	push	temp1
	inc	temp1
	push	temp1
	inc	temp1
	push	temp1
	inc	temp1
	push	temp1
	inc	temp1
	push	temp1
	inc	temp1
	push	temp1
	inc	temp1
	push	temp1
	inc	temp1
	push	temp1
	inc	temp1
	push	temp1
	inc	temp1
	push	temp1
	inc	temp1
	push	temp1
	inc	temp1
	push	temp1
	inc	temp1
	push	temp1
	inc	temp1
	push	temp1
	pop	temp1
	pop	temp1
	pop	temp1
	pop	temp1
	pop	temp1
	pop	temp1
	pop	temp1
	pop	temp1
	pop	temp1
	pop	temp1
	pop	temp1
	pop	temp1
	pop	temp1
	pop	temp1
	pop	temp1
	pop	temp1
	pop	temp1
	pop	temp1
	pop	temp1

	ldi	temp1, 0xff
	out	DDRB, temp1
loop:		
	in	temp1, PORTB
	dec	temp1
	out	PORTB, temp1
	rjmp	loop
